import json

import requests
from shapely import Point

from . import _auth as a
from ._auth import auth


@auth
def get_paleo_coordinates(
    lats,
    lons,
    age,
    model="SETON2012",
    pids=[],
):
    """convenient function to get paleo-coordinates at {age}"""

    if len(lats) != len(lons):
        raise Exception("the length of lats and lons must be the same.")

    pids_len = len(pids)
    if pids_len > 0:
        if pids_len != 1 and pids_len != len(lons):
            raise Exception(
                "the length of pids must be 0, 1 or the length of lons/lats."
            )

    data = {"time": age, "return_null_points": ""}

    # construct the request string for points
    lats_str = ""
    lons_str = ""
    for lat in lats:
        lats_str += f"{lat:3.2f},"
    for lon in lons:
        lons_str += f"{lon:3.2f},"

    data["lats"] = lats_str[:-1]
    data["lons"] = lons_str[:-1]

    # construct the request string for plates ids
    if pids_len == 1:
        data["pid"] = f"{int(pids[0])}"
    elif pids_len == len(lons):
        pids_str = ""
        for pid_ in pids:
            pids_str += f"{int(pid_)},"
        data["pids"] = pids_str[:-1]

    data["model"] = model

    ret = requests.post(
        a.server_url + "/reconstruct/reconstruct_points/",
        data=data,
        verify=True,
        proxies={"http": a.proxy},
    )
    # print(ret.text)

    data = json.loads(str(ret.text))
    # print(data)
    lats_r = []
    lons_r = []
    for c in data["coordinates"]:
        if not c:
            lats_r.append(float("NAN"))
            lons_r.append(float("NAN"))
        else:
            lats_r.append(c[1])
            lons_r.append(c[0])

    return {"lats": lats_r, "lons": lons_r}


@auth
def reconstruct_points(
    points: list[Point],
    age,
    model="SETON2012",
    pids=[],
):
    """get paleo-coordinates for a list of shapely points, return new shapely points"""
    lats = [p.y for p in points]
    lons = [p.x for p in points]

    data = get_paleo_coordinates(lats, lons, age, model, pids)

    return [Point(x, y) for x, y in zip(data["lons"], data["lats"])]



#!/usr/bin/env python3
import sys

import cartopy.crs as ccrs
import matplotlib.pyplot as plt

sys.path.insert(0, "../src/")
from common import OUTPUT_DIR, save_fig
from shapely.geometry import MultiLineString

from gplates_ws_proxy import PlateModel

# dev test
# export GWS_URL=http://localhost:18000/
# micromamba run -n gplates-ws-example ./plot_topological_plate_boundaries.py


def main(show=True):
    model = PlateModel("Muller2019")
    time = 10
    topology_10 = model.get_topology(time)

    polylines = topology_10.get_plate_boundaries(return_format="shapely")

    fig = plt.figure(figsize=(12, 6), dpi=120)
    ax = plt.axes(projection=ccrs.Robinson())

    ax.set_global()
    ax.gridlines()

    for line in polylines:
        if isinstance(line, MultiLineString):
            for g in line.geoms:
                ax.plot(*g.xy, transform=ccrs.PlateCarree(), color="blue")
        else:
            ax.plot(*line.xy, transform=ccrs.PlateCarree(), color="blue")
    plt.title(f"{time} Ma")
    if show:
        plt.show()
    else:
        save_fig(__file__)


if __name__ == "__main__":
    if len(sys.argv) == 2 and sys.argv[1] == "save":
        main(show=False)
    else:
        main(show=True)



import math

from . import quaternions, rotation


def get_third_vertex(point_a, point_b, height=0.05):
    """Given two points, get third point of triangle.

    :params point_a: {"lon": lon, "lat": lat} in radian
    :params point_b: {"lon": lon, "lat": lat} in radian
    :params height: in radian

    :returns: {"lon": lon, "lat": lat} in radian
    """

    assert not (
        math.isclose(point_a["lon"], point_b["lon"])
        and math.isclose(point_a["lat"], point_b["lat"])
    )

    PA_xyz = quaternions.lat_lon_to_cart(point_a["lat"], point_a["lon"])
    PB_xyz = quaternions.lat_lon_to_cart(point_b["lat"], point_b["lon"])

    # another method to get middle point
    AB_middle_xyz = (
        (PA_xyz[0] + PB_xyz[0]) / 2,
        (PA_xyz[1] + PB_xyz[1]) / 2,
        (PA_xyz[2] + PB_xyz[2]) / 2,
    )
    AB_middle_xyz = quaternions.normalize(AB_middle_xyz)
    middle_lat, middle_lon = quaternions.cart_to_lat_lon(
        AB_middle_xyz[0], AB_middle_xyz[1], AB_middle_xyz[2]
    )

    """
    # get middle point
    axis, angle = rotation.find_axis_and_angle(
        [PA_lon_lat[1], PA_lon_lat[0]], [PB_lon_lat[1], PB_lon_lat[0]]
    )
    middle_lat, middle_lon = rotation.rotate(
        [PA_lon_lat[1], PA_lon_lat[0]], axis, angle / 2.0
    )
    """

    # get rotation pole
    AB = (PA_xyz[0] - PB_xyz[0], PA_xyz[1] - PB_xyz[1], PA_xyz[2] - PB_xyz[2])
    AB = quaternions.normalize(AB)
    pole_lat, pole_lon = quaternions.cart_to_lat_lon(AB[0], AB[1], AB[2])

    lat, lon = rotation.rotate(
        [middle_lat, middle_lon],
        [pole_lat, pole_lon],
        height,
    )

    return {"lon": lon, "lat": lat}


if __name__ == "__main__":
    a = {"lon": -121.0, "lat": 15.0}
    b = {"lon": -121.0, "lat": -15.0}

    with open("ring.gmt", "w+") as f:
        f.write(f"{a['lon']:.2f} {a['lat']:.2f}\n")
        f.write(f"{b['lon']:.2f} {b['lat']:.2f}\n")
        for angle in range(360):
            third_vertex = get_third_vertex(
                {"lon": math.radians(a["lon"]), "lat": math.radians(a["lat"])},
                {"lon": math.radians(b["lon"]), "lat": math.radians(b["lat"])},
                math.radians(angle),
            )
            f.write(
                f"{math.degrees(third_vertex['lon']):.2f} {math.degrees(third_vertex['lat']):.2f}\n"
            )

    print("done")



import argparse
import json
import sys

from .plate_model import PlateModel


class ArgParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write(f"error: {message}\n")
        self.print_help()
        sys.exit(1)


def main():
    parser = ArgParser()

    subparser = parser.add_subparsers(dest="command")
    ls_cmd = subparser.add_parser("ls")
    get_cmd = subparser.add_parser("get")

    ls_cmd.add_argument("-m", "--rotation-model", type=str, dest="rotation_model")

    get_cmd.add_argument("-m", "--rotation-model", type=str, dest="rotation_model")

    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(0)

    args = parser.parse_args()

    if args.command == "ls":
        if args.rotation_model == None:
            print("Listing all the rotation models....")
            models = PlateModel.list_models()
            print(json.dumps(models))
        else:
            print(f"Listing all the layers in rotation model {args.rotation_model}")
            model_details = PlateModel(args.rotation_model).get_cfg()
            print(json.dumps(model_details, indent=4))

    elif args.command == "get":
        if args.rotation_model:
            model_details = PlateModel(args.rotation_model).get_cfg()
            print(json.dumps(model_details, indent=4))
        else:
            print(f"get something")


if __name__ == "__main__":
    main()



import math

from importer import *

from gplates_ws_proxy import geometry_utils


def test_cross_dateline():
    assert (
        geometry_utils.test_line_segment_cross_dateline_in_degrees(45, 56, 0) == False
    )
    assert (
        geometry_utils.test_line_segment_cross_dateline_in_degrees(170, -170, 0) == True
    )
    assert (
        geometry_utils.test_line_segment_cross_dateline(
            -3.1185386445524252, 3.1306819004686584, 0
        )
        == True
    )

    assert (
        geometry_utils.test_line_segment_cross_dateline(
            3.141592653589793, 3.087522261495965, 0
        )
        == False
    )
    assert (
        geometry_utils.test_line_segment_cross_dateline(
            -3.141592653589793, 3.087522261495965, 0
        )
        == True
    )


def test_intersection_of_great_circles():
    intersections = geometry_utils.find_two_great_circles_intersections(
        {"lon": 0, "lat": 0},
        {"lon": 0, "lat": 90},
        {"lon": 0, "lat": 0},
        {"lon": 90, "lat": 0},
    )
    assert math.isclose(intersections[0]["lat"], 0)
    assert math.isclose(intersections[0]["lon"], 180)
    assert math.isclose(intersections[1]["lat"], 0)
    assert math.isclose(intersections[1]["lon"], 0)
    print(intersections)
    intersections = geometry_utils.find_two_great_circles_intersections(
        {"lon": -35.8744, "lat": 13.9450},
        {"lon": 20.6730, "lat": 4.6999},
        {"lon": 1.5714, "lat": 28.5002},
        {"lon": -16.5855, "lat": -4.9771},
    )
    print(intersections)
    assert math.isclose(intersections[0]["lat"], -10.804293538655628)
    assert math.isclose(intersections[0]["lon"], 171.3465458795531)
    assert math.isclose(intersections[1]["lat"], 10.804293538655628)
    assert math.isclose(intersections[1]["lon"], -8.653454120446886)


if __name__ == "__main__":
    test_cross_dateline()
    test_intersection_of_great_circles()



import json

from importer import *

from gplates_ws_proxy import subduction_teeth

lons = [-43.1124, -6.1282, 5.0676, 18.7966, 30.2718, 40.4890, 52.3162]
lats = [11.3974, 6.6426, 10.8345, 9.1520, 6.0869, 9.1520, 6.9207]


def test_subduction_teeth():
    """
    teeth = subduction_teeth.get_subduction_teeth(
        [math.radians(lon) for lon in lons],
        [math.radians(lat) for lat in lats],
        geojson=True,
        return_degrees=True,
    )
    """
    teeth = subduction_teeth.get_subduction_teeth_in_degrees(
        lons, lats, base_length=1, spacing=0.5, height=0.5, geojson=True
    )

    assert len(teeth) > 0


if __name__ == "__main__":
    teeth = subduction_teeth.get_subduction_teeth_in_degrees(
        lons,
        lats,
        base_length=1,
        spacing=0.5,
        height=0.5,
        geojson=True,
        polarity=subduction_teeth.Polarity.LEFT,
    )

    NO_GEOMETRY_COLLECTION = True

    print(len(teeth))

    data = {"type": "FeatureCollection"}
    data["features"] = []

    if NO_GEOMETRY_COLLECTION:
        for tooth in teeth:
            feature = {"type": "Feature", "properties": {}}
            feature["geometry"] = tooth
            data["features"].append(feature)
    else:
        feature = {"type": "Feature", "properties": {}}
        feature["geometry"] = {"type": "GeometryCollection", "geometries": teeth}
        data["features"].append(feature)

    line_feature = {"type": "Feature", "properties": {}}
    line_feature["geometry"] = {
        "type": "LineString",
        "coordinates": list(zip(lons, lats)),
    }

    data["features"].append(line_feature)

    with open("triangles.json", "w") as outfile:
        json.dump(data, outfile, indent=4)

    print("Done. Teeth have been saved to triangles.json.")



import sys
from pathlib import Path

import cartopy.crs as ccrs
import matplotlib.pyplot as plt

sys.path.insert(0, "../src/")


from gplates_ws_proxy import coastlines

OUTPUT_DIR = "output"

Path(OUTPUT_DIR).mkdir(parents=True, exist_ok=True)


def get_basemap_with_coastlines(model="Muller2019", crs=ccrs.Robinson(), time=140):
    coastlines_shapely = coastlines.get_paleo_coastlines(
        time=time, model=model, format="shapely"
    )

    fig = plt.figure(figsize=(12, 6), dpi=120)
    ax = plt.axes(projection=crs)

    ax.set_global()
    ax.gridlines()

    ax.add_geometries(
        coastlines_shapely,
        crs=ccrs.PlateCarree(),
        facecolor="grey",
        edgecolor="none",
        alpha=0.5,
    )
    return ax


def save_fig(filename):
    output_file = f"{OUTPUT_DIR}/{Path(filename).stem}.png"
    plt.gcf().savefig(output_file, dpi=120, bbox_inches="tight")  # transparent=True)
    print(f"Done! The {output_file} has been saved.")
    plt.close(plt.gcf())



import json

import requests
import shapely
from shapely.geometry import shape

from . import _auth as a
from ._auth import auth
from .coastlines import get_paleo_coastlines
from .paleoearth import get_paleo_labels
from .reconstruction import get_paleo_coordinates, reconstruct_points
from .topology import Topology, get_subduction_zones


class PlateModel:
    def __init__(self, name="MULLER2019"):
        self.name = name

    @auth
    @staticmethod
    def list_models():
        """return a list which contains all the available rotation models"""
        ret = requests.get(
            a.server_url + "/model/list",
            verify=True,
            proxies={"http": a.proxy},
        )
        return json.loads(str(ret.text))

    @auth
    def get_cfg(self):
        """Given a rotation model name, return the details about the model"""
        params = {"model": self.name}

        ret = requests.get(
            a.server_url + f"/model/show",
            params=params,
            verify=True,
            proxies={"http": a.proxy},
        )

        return json.loads(str(ret.text))

    def reconstruct(
        self,
        lats: list[float],
        lons: list[float],
        age: float,
        pids: list[int] = [],
    ) -> dict[str, list]:
        """reconstruct a list of lats and lons to age

        :returns: for example, {'lats':[1.1,2.2], 'lons':[2.1,2.3]}
        """
        return get_paleo_coordinates(lats, lons, age, model=self.name, pids=pids)

    def get_paleo_labels(self):
        """get a list of paleo-labels

        :returns: a dict object, {"names": names, "lons": lons, "lats": lats}
        """
        return get_paleo_labels()

    def get_subduction_zones(self, time):
        """"""
        return get_subduction_zones(self.name, time)

    def get_topology(self, time):
        """"""
        return Topology(self.name, time)

    def get_coastlines(
        self,
        time: float,
        format="geojson",
        facecolor="lime",
        edgecolor="none",
        alpha: float = 0.5,
        extent=(-180, 180, -90, 90),
        wrap: bool = True,
        central_meridian: float = 0.0,
        anchor_plate_id: int = 0,
        min_area: float = None,
    ):
        """"""
        return get_paleo_coastlines(
            time=time,
            model=self.name,
            format=format,
            facecolor=facecolor,
            edgecolor=edgecolor,
            alpha=alpha,
            extent=extent,
            wrap=wrap,
            central_meridian=central_meridian,
            anchor_plate_id=anchor_plate_id,
            min_area=min_area,
        )


def reconstruct_shapely_points(
    model: PlateModel, points: list[shapely.Point], time: float, pids: list[int] = []
) -> list[shapely.Point]:
    """reconstruct a list of shapely points to {time}Ma"""
    return reconstruct_points(points, time, model=model.name, pids=pids)



from shapely.geometry import MultiPolygon, Polygon, shape


def geojson_to_shapely(json_data):
    ret = []
    for feature in json_data["features"]:
        s = shape(feature["geometry"])
        if isinstance(s, Polygon) or isinstance(s, MultiPolygon):
            ret.append(s.buffer(0))
        else:
            ret.append(s)

    return ret



#!/usr/bin/env python3

import sys
from pathlib import Path

import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import shapely

sys.path.insert(0, "../src/")
from common import OUTPUT_DIR

from gplates_ws_proxy import PlateModel, reconstruct_shapely_points

# dev test
# export GWS_URL=http://localhost:18000/
# micromamba run -n gplates-ws-example ./reconstruct_shapely_points.py


def main(show=True):
    lats = [50, 10, 50]
    lons = [-100, 160, 100]

    points = [shapely.Point(x, y) for x, y in zip(lons, lats)]

    model = PlateModel("Muller2019")
    time = 100
    paleo_points = reconstruct_shapely_points(model, points, time)
    print(paleo_points)

    # plot the points
    crs = ccrs.Robinson(central_longitude=0.0, globe=None)
    fig = plt.figure(figsize=(12, 6), dpi=120)
    ax = plt.axes(projection=crs)
    ax.gridlines()
    ax.set_global()
    ax.coastlines()

    for idx, p in enumerate(paleo_points):
        ax.text(p.x, p.y, str(idx) + "P", transform=ccrs.PlateCarree(), fontsize=20)

    for idx, p in enumerate(points):
        ax.text(p.x, p.y, str(idx), transform=ccrs.PlateCarree(), fontsize=20)

    ax.scatter(
        [point.x for point in paleo_points],
        [point.y for point in paleo_points],
        transform=ccrs.PlateCarree(),
    )

    ax.scatter(
        [point.x for point in points],
        [point.y for point in points],
        color="red",
        transform=ccrs.PlateCarree(),
    )
    plt.title(f"{time} Ma", fontsize=20)

    if show:
        plt.show()
    else:
        output_file = f"{OUTPUT_DIR}/{Path(__file__).stem}.png"
        fig.savefig(output_file, bbox_inches="tight")
        plt.close()
        print(f"Done! The {output_file} has been saved.")


if __name__ == "__main__":
    if len(sys.argv) == 2 and sys.argv[1] == "save":
        main(show=False)
    else:
        main(show=True)



import requests
from shapely.geometry import shape

from . import _auth as a
from ._auth import auth


@auth
def get_paleo_coastlines(
    time: float,
    model="MULLER2022",
    format="geojson",
    facecolor="lime",
    edgecolor="none",
    alpha: float = 0.5,
    extent=(-180, 180, -90, 90),
    wrap: bool = True,
    central_meridian: float = 0.0,
    anchor_plate_id: int = 0,
    min_area: float = None,
):
    """Get paleo-coastlines

    By default, the polygons are wrapped along (180/-180). If you would like to wrap them
    at other locations, be careful that some plotting packages might not work well with them.

    :param time: the input paleo age
    :param model: the name of rotation model
    :param format: the return data format, such as geojson, shapely, png
    :param facecolor: face color -- only for png format
    :param edgecolor: edge color -- only for png format
    :param alpha: alpha -- only for png format
    :param extent: (left, right, bottom, top) -- only for png format
    :param anchor_plate_id: anchor plate id
    :param central_meridian: central meridian
    :param wrap: flag to indicate if wrap the polygons along dateline
    :param min_area: only returns polygons with a larger area

    :returns: paleo-coastlines
    :rtype: geojson

    """

    params = {
        "time": time,
        "model": model,
        "anchor_plate_id": anchor_plate_id,
        "wrap": wrap,
        "central_meridian": central_meridian,
    }

    if min_area is not None:
        params["min_area"] = min_area

    if extent is not None:
        params["extent"] = f"{extent[0]},{extent[1]},{extent[2]},{extent[3]}"

    if format == "png":
        params["fmt"] = "png"
        params["facecolor"] = facecolor
        params["edgecolor"] = edgecolor
        params["alpha"] = alpha
        headers = {
            "Accept": "image/png",
        }
    else:
        headers = {
            "Accept": "application/json",
        }

    ret = requests.get(
        a.server_url + "/reconstruct/coastlines/",
        params=params,
        verify=True,
        headers=headers,
        proxies={"http": a.proxy},
    )

    if ret.status_code in [200, 201]:
        if format == "shapely":
            json_data = ret.json()
            geoms = [
                shape(feature["geometry"]).buffer(0)
                for feature in json_data["features"]
            ]
            return geoms
        elif format == "png":
            return ret.content
        else:
            json_data = ret.json()
            return json_data
    else:
        raise Exception("Failed to get paleo-coastlines.")



#!/usr/bin/env python3

import matplotlib.pyplot as plt
import moviepy.editor as mpy
from common import OUTPUT_DIR, get_basemap_with_coastlines, save_fig


def main():
    for time in range(0, 141, 10):
        get_basemap_with_coastlines(time=time)
        plt.title(f"{time} Ma")
        save_fig(f"paleo-coastlines-{time}-Ma.png")

    frame_list = [
        f"{OUTPUT_DIR}/paleo-coastlines-{time}-Ma.png" for time in range(0, 141, 10)
    ]
    # print(frame_list)
    clip = mpy.ImageSequenceClip(frame_list, fps=4)

    clip.write_videofile(
        f"{OUTPUT_DIR}/coastlines.mp4",
        codec="libx264",
        # audio_codec='aac',
        ffmpeg_params=["-s", "1440x720", "-pix_fmt", "yuv420p"],
    )  # LOOK HERE!!!! give image size here(the numbers must divide by 2)
    print("video has been created!")


if __name__ == "__main__":
    main()



import math

from .rotation import cross


def test_line_segment_cross_dateline_in_degrees(
    point_a_lon, point_b_lon, central_meridian
):
    """same as test_line_segment_cross_dateline except all arguments are in degrees"""
    return test_line_segment_cross_dateline(
        math.radians(point_a_lon),
        math.radians(point_b_lon),
        math.radians(central_meridian),
    )


def test_line_segment_cross_dateline(point_a_lon, point_b_lon, central_meridian):
    """Return True if the line segment cross dateline, otherwise False.
    All parameters are in radians.
    """
    dateline_lon = central_meridian + math.pi

    a_to_dateline = abs(point_a_lon - dateline_lon) % (math.pi * 2)
    b_to_dateline = abs(point_b_lon - dateline_lon) % (math.pi * 2)
    a_to_b = abs(point_a_lon - point_b_lon) % (math.pi * 2)

    assert not math.isclose(
        a_to_b, math.pi
    )  # two points are 180 degree apart along longitude axis. cannot decide if cross dateline

    if a_to_dateline > math.pi:
        a_to_dateline = 2 * math.pi - a_to_dateline
    if b_to_dateline > math.pi:
        b_to_dateline = 2 * math.pi - b_to_dateline
    if a_to_b > math.pi:
        a_to_b = 2 * math.pi - a_to_b

    # print(a_to_dateline, b_to_dateline, a_to_b)

    if math.isclose(a_to_dateline, 0.0) or math.isclose(b_to_dateline, 0.0):
        # print(point_a_lon, point_b_lon)
        if abs(point_a_lon - point_b_lon) < math.pi:
            return False
        else:
            return True

    if math.isclose(a_to_dateline + b_to_dateline, a_to_b):
        return True

    if a_to_dateline + b_to_dateline > a_to_b:
        return False
    else:
        return True


def lat_lon_to_xyz(lat, lon):
    """all in radians"""
    x = math.cos(lat) * math.cos(lon)
    y = math.cos(lat) * math.sin(lon)
    z = math.sin(lat)
    return x, y, z


def xyz_to_lat_lon(x, y, z):
    """in radians"""
    lat = math.asin(z) * 180.0 / math.pi
    lon = math.atan2(y, x) * 180.0 / math.pi
    return lat, lon


def find_two_great_circles_intersections(point_a, point_b, point_c, point_d):
    """return the two intersection points"""
    # to gradians
    pa_lat_rad = math.radians(point_a["lat"])
    pa_lon_rad = math.radians(point_a["lon"])
    pb_lat_rad = math.radians(point_b["lat"])
    pb_lon_rad = math.radians(point_b["lon"])

    # to gradians
    pc_lat_rad = math.radians(point_c["lat"])
    pc_lon_rad = math.radians(point_c["lon"])
    pd_lat_rad = math.radians(point_d["lat"])
    pd_lon_rad = math.radians(point_d["lon"])

    # get xyz coordinates
    xa, ya, za = lat_lon_to_xyz(pa_lat_rad, pa_lon_rad)
    xb, yb, zb = lat_lon_to_xyz(pb_lat_rad, pb_lon_rad)
    xc, yc, zc = lat_lon_to_xyz(pc_lat_rad, pc_lon_rad)
    xd, yd, zd = lat_lon_to_xyz(pd_lat_rad, pd_lon_rad)

    # Get normal to planes containing great circles
    N1 = cross([xa, ya, za], [xb, yb, zb])
    N2 = cross([xc, yc, zc], [xd, yd, zd])

    # Find line of intersection between two planes
    L = cross(N1, N2)

    # Find two intersection points
    X1 = [x / math.sqrt(L[0] ** 2 + L[1] ** 2 + L[2] ** 2) for x in L]
    X2 = [-x for x in X1]
    lat1, lon1 = xyz_to_lat_lon(X1[0], X1[1], X1[2])
    lat2, lon2 = xyz_to_lat_lon(X2[0], X2[1], X2[2])

    return {"lat": lat1, "lon": lon1}, {"lat": lat2, "lon": lon2}



#!/usr/bin/env python3

import sys

import cartopy.crs as ccrs
import matplotlib.pyplot as plt

sys.path.insert(0, "../src/")
from common import OUTPUT_DIR, get_basemap_with_coastlines, save_fig

from gplates_ws_proxy import plate_model, subduction_teeth

# dev test
# export GWS_URL=http://localhost:18000/
# micromamba run -n gplates-ws-example ./plot_subduction_zones.py


def plot_teeth(ax, lons, lats, polarity):
    teeth = subduction_teeth.get_subduction_teeth_in_degrees(
        lons,
        lats,
        base_length=2,
        spacing=1,
        height=2,
        polarity=polarity,
        central_meridian=0,
    )
    for tooth in teeth:
        x = [t["lon"] for t in tooth]
        y = [t["lat"] for t in tooth]
        # print(x)
        ax.fill(x, y, transform=ccrs.PlateCarree())


def main(show=True):
    time = 0
    model_name = "Merdith2021"

    ax = get_basemap_with_coastlines(model=model_name, time=time)

    model = plate_model.PlateModel(model_name)

    subduction_zones = model.get_subduction_zones(time)

    for feature in subduction_zones["features"]:
        geom = feature["geometry"]
        polarity = subduction_teeth.Polarity.UNKNOWN
        _polarity = feature["properties"]["polarity"]
        if _polarity.lower() == "right":
            polarity = subduction_teeth.Polarity.RIGHT
        elif _polarity.lower() == "left":
            polarity = subduction_teeth.Polarity.LEFT

        if geom["type"] == "LineString":
            lats = [lon_lat[1] for lon_lat in geom["coordinates"]]
            lons = [lon_lat[0] for lon_lat in geom["coordinates"]]
            ax.plot(lons, lats, transform=ccrs.PlateCarree())
            plot_teeth(ax, lons, lats, polarity)

        elif geom["type"] == "MultiLineString":
            for line in geom["coordinates"]:
                lats = [lon_lat[1] for lon_lat in line]
                lons = [lon_lat[0] for lon_lat in line]
                ax.plot(lons, lats, transform=ccrs.PlateCarree())
                plot_teeth(ax, lons, lats, polarity)

    plt.title(f"{time} Ma", fontsize=20)

    if show:
        plt.show()
    else:
        save_fig(__file__)


if __name__ == "__main__":
    if len(sys.argv) == 2 and sys.argv[1] == "save":
        main(show=False)
    else:
        main(show=True)



from importer import *
from shapely import Point


def test_reconstruct_points():
    lats = [50, 10, 50]
    lons = [-100, 160, 100]

    points = [Point(x, y) for x, y in zip(lons, lats)]

    model = gplates.PlateModel("Muller2019")
    paleo_points = gplates.reconstruct_shapely_points(model, points, 100)
    print(paleo_points)

    paleo_points = model.reconstruct(lats, lons, 100)
    print(paleo_points)
